@@ -59,7 +59,9 @@ U_STABLE void U_EXPORT2 _stopDecode(const void *context,
     if (chars && length)
     {
         const int size = stop->src_length - length + 1;
-        const int len = length < sizeof(stop->chars) - 1 ? length : sizeof(stop->chars) - 1;
+        const size_t len = (size_t) length < sizeof(stop->chars) - 1
+            ? (size_t) length
+            : sizeof(stop->chars) - 1;
 
         strncpy(stop->chars, chars, len); stop->chars[len] = '\0';
         stop->error_position = -1;
@@ -86,6 +88,13 @@ EXPORT ICUException::ICUException()
     msg = NULL;
 }
 
+EXPORT ICUException::ICUException(const ICUException &e)
+    : code(e.code), msg(e.msg)
+{
+    Py_XINCREF(code);
+    Py_XINCREF(msg);
+}
+
 EXPORT ICUException::ICUException(UErrorCode status)
 {
     PyObject *messages = PyObject_GetAttrString(PyExc_ICUError, "messages");
@@ -113,7 +122,7 @@ EXPORT ICUException::ICUException(UParseError &pe, UErrorCode status)
     PyObject *tuple = PyTuple_New(5);
 
     ICUException::code = PyInt_FromLong((long) status);
-    
+
     PyTuple_SET_ITEM(tuple, 0, PyObject_GetItem(messages, code));
     PyTuple_SET_ITEM(tuple, 1, PyInt_FromLong(pe.line));
     PyTuple_SET_ITEM(tuple, 2, PyInt_FromLong(pe.offset));
@@ -139,62 +148,106 @@ EXPORT PyObject *ICUException::reportError()
         PyErr_SetObject(PyExc_ICUError, tuple);
         Py_DECREF(tuple);
     }
-        
+
     return NULL;
 }
 
 
-EXPORT PyObject *PyUnicode_FromUnicodeString(UnicodeString *string)
+EXPORT PyObject *PyUnicode_FromUnicodeString(const UnicodeString *string)
 {
     if (!string)
     {
         Py_INCREF(Py_None);
         return Py_None;
     }
-    else if (sizeof(Py_UNICODE) == sizeof(UChar))
-        return PyUnicode_FromUnicode((const Py_UNICODE *) string->getBuffer(),
-                                     (int) string->length());
-    else
-    {
-        int len = string->length();
-        PyObject *u = PyUnicode_FromUnicode(NULL, len);
 
-        if (u)
-        {
-            Py_UNICODE *pchars = PyUnicode_AS_UNICODE(u);
-            const UChar *chars = string->getBuffer();
-
-            for (int i = 0; i < len; i++)
-                pchars[i] = chars[i];
-        }        
-
-        return u;
-    }
+    return PyUnicode_FromUnicodeString(string->getBuffer(), string->length());
 }
 
-EXPORT PyObject *PyUnicode_FromUnicodeString(const UChar *chars, int size)
+EXPORT PyObject *PyUnicode_FromUnicodeString(const UChar *utf16, int len16)
 {
-    if (!chars)
+    if (!utf16)
     {
         Py_INCREF(Py_None);
         return Py_None;
     }
+
+#if PY_VERSION_HEX < 0x03030000
     else if (sizeof(Py_UNICODE) == sizeof(UChar))
-        return PyUnicode_FromUnicode((const Py_UNICODE *) chars, size);
+        return PyUnicode_FromUnicode((const Py_UNICODE *) utf16, len16);
     else
     {
-        PyObject *u = PyUnicode_FromUnicode(NULL, size);
+        int32_t len32 = u_countChar32(utf16, len16);
+        PyObject *u = PyUnicode_FromUnicode(NULL, len32);
 
         if (u)
         {
             Py_UNICODE *pchars = PyUnicode_AS_UNICODE(u);
+            UErrorCode status = U_ZERO_ERROR;
 
-            for (int i = 0; i < size; i++)
-                pchars[i] = chars[i];
+            u_strToUTF32((UChar32 *) pchars, len32, NULL,
+                         utf16, len16, &status);
+            if (U_FAILURE(status))
+            {
+                Py_DECREF(u);
+                return ICUException(status).reportError();
+            }
         }
-        
+
         return u;
     }
+#else
+    {
+        int32_t len32 = 0;
+        UChar32 max_char = 0;
+
+        for (int32_t i = 0; i < len16;) {
+            UChar32 cp;
+
+            U16_NEXT(utf16, i, len16, cp);
+            max_char |= cp;  // we only care about the leftmost bit
+            len32 += 1;
+        }
+
+        PyObject *result = PyUnicode_New(len32, max_char);
+
+        if (result == NULL)
+            return NULL;
+
+        switch (PyUnicode_KIND(result)) {
+          case PyUnicode_1BYTE_KIND:
+            // note: len16 == len32
+            for (int32_t i = 0; i < len32; ++i)
+                PyUnicode_1BYTE_DATA(result)[i] = (Py_UCS1) (utf16[i]);
+            break;
+
+          case PyUnicode_2BYTE_KIND:
+            // note: len16 == len32
+            u_memcpy(PyUnicode_2BYTE_DATA(result), utf16, len16);
+            break;
+
+          case PyUnicode_4BYTE_KIND: {
+            UErrorCode status = U_ZERO_ERROR;
+
+            // note: len16 > len32 (len32 is at least half of len16)
+            u_strToUTF32((UChar32 *) PyUnicode_4BYTE_DATA(result), len32, NULL,
+                         utf16, len16, &status);
+            if (U_FAILURE(status))
+            {
+                Py_DECREF(result);
+                return ICUException(status).reportError();
+            }
+            break;
+          }
+
+          default:
+            Py_DECREF(result);
+            return NULL;
+        }
+
+        return result;
+    }
+#endif
 }
 
 EXPORT UnicodeString &PyBytes_AsUnicodeString(PyObject *object,
@@ -284,6 +337,7 @@ EXPORT UnicodeString &PyObject_AsUnicodeString(PyObject *object,
 {
     if (PyUnicode_Check(object))
     {
+#if PY_VERSION_HEX < 0x03030000
         if (sizeof(Py_UNICODE) == sizeof(UChar))
             string.setTo((const UChar *) PyUnicode_AS_UNICODE(object),
                          (int32_t) PyUnicode_GET_SIZE(object));
@@ -291,22 +345,63 @@ EXPORT UnicodeString &PyObject_AsUnicodeString(PyObject *object,
         {
             int32_t len = (int32_t) PyUnicode_GET_SIZE(object);
             Py_UNICODE *pchars = PyUnicode_AS_UNICODE(object);
-            UChar *chars = new UChar[len * 3];
-            UErrorCode status = U_ZERO_ERROR;
-            int32_t dstLen;
 
-            u_strFromUTF32(chars, len * 3, &dstLen,
-                           (const UChar32 *) pchars, len, &status);
-
-            if (U_FAILURE(status))
-            {
-                delete[] chars;
-                throw ICUException(status);
-            }
-
-            string.setTo((const UChar *) chars, (int32_t) dstLen);
-            delete[] chars;
+            string = UnicodeString::fromUTF32((const UChar32 *) pchars, len);
         }
+#else
+        PyUnicode_READY(object);
+
+        switch (PyUnicode_KIND(object)) {
+          case PyUnicode_WCHAR_KIND: {  // this code path should be deprecated
+              if (sizeof(Py_UNICODE) == sizeof(UChar))
+              {
+                  Py_ssize_t len;
+                  Py_UNICODE *pchars = PyUnicode_AsUnicodeAndSize(object, &len);
+
+                  string.setTo((const UChar *) pchars, len);
+              }
+              else
+              {
+                  Py_ssize_t len;
+                  Py_UNICODE *pchars = PyUnicode_AsUnicodeAndSize(object, &len);
+
+                  string = UnicodeString::fromUTF32(
+                      (const UChar32 *) pchars, len);
+              }
+              break;
+          }
+
+          case PyUnicode_1BYTE_KIND: {
+              Py_ssize_t len = PyUnicode_GET_LENGTH(object);
+              Py_UCS1 *pchars = PyUnicode_1BYTE_DATA(object);
+              UChar *chars = string.getBuffer(len);
+
+              if (chars != NULL)
+              {
+                  for (int i = 0; i < len; ++i)
+                      chars[i] = (UChar) pchars[i];
+                  string.releaseBuffer(len);
+              }
+              break;
+          }
+
+          case PyUnicode_2BYTE_KIND: {
+              Py_ssize_t len = PyUnicode_GET_LENGTH(object);
+              Py_UCS2 *pchars = PyUnicode_2BYTE_DATA(object);
+
+              string.setTo((const UChar *) pchars, len);
+              break;
+          }
+
+          case PyUnicode_4BYTE_KIND: {
+              Py_ssize_t len = PyUnicode_GET_LENGTH(object);
+              Py_UCS4 *pchars = PyUnicode_4BYTE_DATA(object);
+
+              string = UnicodeString::fromUTF32((const UChar32 *) pchars, len);
+              break;
+          }
+        }
+#endif
     }
     else if (PyBytes_Check(object))
         PyBytes_AsUnicodeString(object, encoding, mode, string);
