@@ -51,6 +51,12 @@ static int t_script_init(t_script *self, PyObject *args, PyObject *kwds);
 static PyObject *t_script_getName(t_script *self);
 static PyObject *t_script_getShortName(t_script *self);
 static PyObject *t_script_getScriptCode(t_script *self);
+static PyObject *t_script_getCode(PyTypeObject *type, PyObject *arg);
+static PyObject *t_script_getScript(PyTypeObject *type, PyObject *arg);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
+static PyObject *t_script_hasScript(PyTypeObject *type, PyObject *args);
+static PyObject *t_script_getScriptExtensions(PyTypeObject *type, PyObject *arg);
+#endif
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
 static PyObject *t_script_isRightToLeft(t_script *self);
 static PyObject *t_script_isCased(t_script *self);
@@ -58,15 +64,17 @@ static PyObject *t_script_breaksBetweenLetters(t_script *self);
 static PyObject *t_script_getSampleString(t_script *self);
 static PyObject *t_script_getUsage(t_script *self);
 #endif
-static PyObject *t_script_getCode(PyTypeObject *type, PyObject *arg);
-static PyObject *t_script_getScript(PyTypeObject *type, PyObject *arg);
-static PyObject *t_script_hasScript(PyTypeObject *type, PyObject *args);
-static PyObject *t_script_getScriptExtensions(PyTypeObject *type, PyObject *arg);
 
 static PyMethodDef t_script_methods[] = {
     DECLARE_METHOD(t_script, getName, METH_NOARGS),
     DECLARE_METHOD(t_script, getShortName, METH_NOARGS),
     DECLARE_METHOD(t_script, getScriptCode, METH_NOARGS),
+    DECLARE_METHOD(t_script, getCode, METH_O | METH_CLASS),
+    DECLARE_METHOD(t_script, getScript, METH_O | METH_CLASS),
+#if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
+    DECLARE_METHOD(t_script, hasScript, METH_VARARGS | METH_CLASS),
+    DECLARE_METHOD(t_script, getScriptExtensions, METH_O | METH_CLASS),
+#endif
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
     DECLARE_METHOD(t_script, isRightToLeft, METH_NOARGS),
     DECLARE_METHOD(t_script, isCased, METH_NOARGS),
@@ -74,10 +82,6 @@ static PyMethodDef t_script_methods[] = {
     DECLARE_METHOD(t_script, getSampleString, METH_NOARGS),
     DECLARE_METHOD(t_script, getUsage, METH_NOARGS),
 #endif
-    DECLARE_METHOD(t_script, getCode, METH_O | METH_CLASS),
-    DECLARE_METHOD(t_script, getScript, METH_O | METH_CLASS),
-    DECLARE_METHOD(t_script, hasScript, METH_VARARGS | METH_CLASS),
-    DECLARE_METHOD(t_script, getScriptExtensions, METH_O | METH_CLASS),
     { NULL, NULL, 0, NULL }
 };
 
@@ -134,6 +138,159 @@ static PyObject *t_script_getScriptCode(t_script *self)
     return PyInt_FromLong(self->code);
 }
 
+static PyObject *t_script_getCode(PyTypeObject *type, PyObject *arg)
+{
+    charsArg name;
+
+    if (!parseArg(arg, "n", &name))
+    {
+        UScriptCode codes[256];
+        int count;
+
+        STATUS_CALL(count = uscript_getCode(
+            name, codes, sizeof(codes) / sizeof(UScriptCode), &status));
+
+        PyObject *tuple = PyTuple_New(count);
+
+        for (int i = 0; i < count; ++i)
+            PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(codes[i]));
+
+        return tuple;
+    }
+
+    return PyErr_SetArgsError((PyObject *) type, "getCode", arg);
+}
+
+static PyObject *t_script_getScript(PyTypeObject *type, PyObject *arg)
+{
+    UnicodeString *u, _u;
+    int cp;
+
+    if (!parseArg(arg, "S", &u, &_u))
+    {
+        UScriptCode code;
+
+        if (u->countChar32() != 1)
+        {
+            PyObject *tuple = Py_BuildValue(
+                "(sO)", "string must contain only one codepoint", arg);
+
+            PyErr_SetObject(PyExc_ValueError, tuple);
+            Py_DECREF(tuple);
+
+            return NULL;
+        }
+
+        STATUS_CALL(code = uscript_getScript(u->char32At(0), &status));
+
+        return PyObject_CallFunction((PyObject *) type, (char *) "i", code);
+    }
+    if (!parseArg(arg, "i", &cp))
+    {
+        UScriptCode code;
+
+        STATUS_CALL(code = uscript_getScript((UChar32) cp, &status));
+
+        return PyObject_CallFunction((PyObject *) type, (char *) "i", code);
+    }
+
+    return PyErr_SetArgsError((PyObject *) type, "getScript", arg);
+}
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
+static PyObject *t_script_hasScript(PyTypeObject *type, PyObject *args)
+{
+    UnicodeString *u, _u;
+    UScriptCode code;
+    int cp;
+
+    switch (PyTuple_Size(args)) {
+      case 2:
+        if (!parseArgs(args, "Si", &u, &_u, &code))
+        {
+            if (u->countChar32() != 1)
+            {
+                PyObject *tuple = Py_BuildValue(
+                    "(sO)", "string must contain only one codepoint",
+                    PyTuple_GET_ITEM(args, 0));
+
+                PyErr_SetObject(PyExc_ValueError, tuple);
+                Py_DECREF(tuple);
+
+                return NULL;
+            }
+
+            if (uscript_hasScript(u->char32At(0), code))
+                Py_RETURN_TRUE;
+
+            Py_RETURN_FALSE;
+        }
+        if (!parseArgs(args, "ii", &cp, &code))
+        {
+            if (uscript_hasScript((UChar32) cp, code))
+                Py_RETURN_TRUE;
+
+            Py_RETURN_FALSE;
+        }
+        break;
+    }
+
+    return PyErr_SetArgsError((PyObject *) type, "hasScript", args);
+}
+
+static PyObject *t_script_getScriptExtensions(PyTypeObject *type, PyObject *arg)
+{
+    UnicodeString *u, _u;
+    int cp;
+
+    if (!parseArg(arg, "S", &u, &_u))
+    {
+        if (u->countChar32() != 1)
+        {
+            PyObject *tuple = Py_BuildValue(
+                "(sO)", "string must contain only one codepoint", arg);
+
+            PyErr_SetObject(PyExc_ValueError, tuple);
+            Py_DECREF(tuple);
+
+            return NULL;
+        }
+
+        UScriptCode codes[256];
+        int count;
+
+        STATUS_CALL(count = uscript_getScriptExtensions(
+            u->char32At(0), codes, sizeof(codes) / sizeof(UScriptCode),
+            &status));
+
+        PyObject *tuple = PyTuple_New(count);
+
+        for (int i = 0; i < count; ++i)
+            PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(codes[i]));
+
+        return tuple;
+    }
+    if (!parseArg(arg, "i", &cp))
+    {
+        UScriptCode codes[256];
+        int count;
+
+        STATUS_CALL(count = uscript_getScriptExtensions(
+            (UChar32) cp, codes, sizeof(codes) / sizeof(UScriptCode),
+            &status));
+
+        PyObject *tuple = PyTuple_New(count);
+
+        for (int i = 0; i < count; ++i)
+            PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(codes[i]));
+
+        return tuple;
+    }
+
+    return PyErr_SetArgsError((PyObject *) type, "getScriptExtensions", arg);
+}
+#endif
+
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
 static PyObject *t_script_isRightToLeft(t_script *self)
 {
@@ -176,120 +333,6 @@ static PyObject *t_script_getUsage(t_script *self)
 }
 #endif
 
-static PyObject *t_script_getCode(PyTypeObject *type, PyObject *arg)
-{
-    charsArg name;
-
-    if (!parseArg(arg, "n", &name))
-    {
-        UScriptCode codes[256];
-        int count;
-
-        STATUS_CALL(count = uscript_getCode(
-            name, codes, sizeof(codes) / sizeof(UScriptCode), &status));
-
-        PyObject *tuple = PyTuple_New(count);
-
-        for (int i = 0; i < count; ++i)
-            PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(codes[i]));
-
-        return tuple;
-    }
-
-    return PyErr_SetArgsError((PyObject *) type, "getCode", arg);
-}
-
-static PyObject *t_script_getScript(PyTypeObject *type, PyObject *arg)
-{
-    UnicodeString *u, _u;
-
-    if (!parseArg(arg, "S", &u, &_u))
-    {
-        UScriptCode code;
-
-        if (u->length() != 1)
-        {
-            PyObject *tuple = Py_BuildValue("(sO)", "length must be 1", arg);
-
-            PyErr_SetObject(PyExc_ValueError, tuple);
-            Py_DECREF(tuple);
-
-            return NULL;
-        }
-  
-        STATUS_CALL(code = uscript_getScript(u->char32At(0), &status));
-
-        return PyObject_CallFunction((PyObject *) type, (char *) "i", code);
-    }
-
-    return PyErr_SetArgsError((PyObject *) type, "getScript", arg);
-}
-
-static PyObject *t_script_hasScript(PyTypeObject *type, PyObject *args)
-{
-    UnicodeString *u, _u;
-    UScriptCode code;
-
-    switch (PyTuple_Size(args)) {
-      case 2:
-        if (!parseArgs(args, "Si", &u, &_u, &code))
-        {
-            if (u->length() != 1)
-            {
-                PyObject *tuple = Py_BuildValue("(sO)", "length must be 1",
-                                                PyTuple_GET_ITEM(args, 0));
-
-                PyErr_SetObject(PyExc_ValueError, tuple);
-                Py_DECREF(tuple);
-
-                return NULL;
-            }
-  
-            if (uscript_hasScript(u->char32At(0), code))
-                Py_RETURN_TRUE;
-
-            Py_RETURN_FALSE;
-        }
-        break;
-    }
-
-    return PyErr_SetArgsError((PyObject *) type, "hasScript", args);
-}
-
-static PyObject *t_script_getScriptExtensions(PyTypeObject *type, PyObject *arg)
-{
-    UnicodeString *u, _u;
-
-    if (!parseArg(arg, "S", &u, &_u))
-    {
-        if (u->length() != 1)
-        {
-            PyObject *tuple = Py_BuildValue("(sO)", "length must be 1", arg);
-
-            PyErr_SetObject(PyExc_ValueError, tuple);
-            Py_DECREF(tuple);
-
-            return NULL;
-        }
-  
-        UScriptCode codes[256];
-        int count;
-
-        STATUS_CALL(count = uscript_getScriptExtensions(
-            u->char32At(0), codes, sizeof(codes) / sizeof(UScriptCode),
-            &status));
-
-        PyObject *tuple = PyTuple_New(count);
-
-        for (int i = 0; i < count; ++i)
-            PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(codes[i]));
-
-        return tuple;
-    }
-
-    return PyErr_SetArgsError((PyObject *) type, "getScriptExtensions", arg);
-}
-
 
 void _init_script(PyObject *m)
 {
@@ -384,9 +427,7 @@ void _init_script(PyObject *m)
     INSTALL_ENUM(UScriptCode, "LATIN_GAELIC", USCRIPT_LATIN_GAELIC);
     INSTALL_ENUM(UScriptCode, "LEPCHA", USCRIPT_LEPCHA);
     INSTALL_ENUM(UScriptCode, "LINEAR_A", USCRIPT_LINEAR_A);
-    INSTALL_ENUM(UScriptCode, "MANDAIC", USCRIPT_MANDAIC);
     INSTALL_ENUM(UScriptCode, "MANDAEAN", USCRIPT_MANDAEAN);
-    INSTALL_ENUM(UScriptCode, "MEROITIC_HIEROGLYPHS", USCRIPT_MEROITIC_HIEROGLYPHS);
     INSTALL_ENUM(UScriptCode, "MEROITIC", USCRIPT_MEROITIC);
     INSTALL_ENUM(UScriptCode, "ORKHON", USCRIPT_ORKHON);
     INSTALL_ENUM(UScriptCode, "OLD_PERMIC", USCRIPT_OLD_PERMIC);
@@ -448,8 +489,10 @@ void _init_script(PyObject *m)
     INSTALL_ENUM(UScriptCode, "GRANTHA", USCRIPT_GRANTHA);
     INSTALL_ENUM(UScriptCode, "KPELLE", USCRIPT_KPELLE);
     INSTALL_ENUM(UScriptCode, "LOMA", USCRIPT_LOMA);
+    INSTALL_ENUM(UScriptCode, "MANDAIC", USCRIPT_MANDAIC);
     INSTALL_ENUM(UScriptCode, "MENDE", USCRIPT_MENDE);
     INSTALL_ENUM(UScriptCode, "MEROITIC_CURSIVE", USCRIPT_MEROITIC_CURSIVE);
+    INSTALL_ENUM(UScriptCode, "MEROITIC_HIEROGLYPHS", USCRIPT_MEROITIC_HIEROGLYPHS);
     INSTALL_ENUM(UScriptCode, "OLD_NORTH_ARABIAN", USCRIPT_OLD_NORTH_ARABIAN);
     INSTALL_ENUM(UScriptCode, "NABATAEAN", USCRIPT_NABATAEAN);
     INSTALL_ENUM(UScriptCode, "PALMYRENE", USCRIPT_PALMYRENE);
