@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2009-2011 Open Source Applications Foundation.
+ * Copyright (c) 2009-2015 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -178,12 +178,12 @@ void PythonTransliterator::handleTransliterate(Replaceable& text,
 static int t_utransposition_init(t_utransposition *self,
                                  PyObject *args, PyObject *kwds)
 {
-    static char *kwnames[] = {
+    static const char *kwnames[] = {
         "contextStart", "contextLimit", "start", "limit", NULL
     };
     int contextStart = 0, contextLimit = 0, start = 0, limit = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iiii", kwnames,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iiii", (char **) kwnames,
                                      &contextStart, &contextLimit,
                                      &start, &limit))
         return -1;
@@ -273,25 +273,25 @@ static int t_utransposition_setStart(t_utransposition *self,
 }
 
 static PyGetSetDef t_utransposition_properties[] = {
-    {"contextLimit",
+    {(char *) "contextLimit",
      (getter) t_utransposition_getContextLimit,
      (setter) t_utransposition_setContextLimit,
-     "Ending index, exclusive, of the context to be considered for a transliteration operation.",
+     (char *) "Ending index, exclusive, of the context to be considered for a transliteration operation.",
      NULL},
-    {"contextStart",
+    {(char *) "contextStart",
      (getter) t_utransposition_getContextStart,
      (setter) t_utransposition_setContextStart,
-     "Beginning index, inclusive, of the context to be considered for a transliteration operation.",
+     (char *) "Beginning index, inclusive, of the context to be considered for a transliteration operation.",
      NULL},
-    {"limit",
+    {(char *) "limit",
      (getter) t_utransposition_getLimit,
      (setter) t_utransposition_setLimit,
-     "Ending index, exclusive, of the text to be transliterated.",
+     (char *) "Ending index, exclusive, of the text to be transliterated.",
      NULL},
-    {"start",
+    {(char *) "start",
      (getter) t_utransposition_getStart,
      (setter) t_utransposition_setStart,
-     "Beginning index, inclusive, of the text to be transliterated.",
+     (char *) "Beginning index, inclusive, of the text to be transliterated.",
      NULL},
     {NULL}  /* Sentinel */
 };
@@ -391,12 +391,12 @@ static PyObject *t_transliterator_transliterate(t_transliterator *self,
         }
         break;
       case 2:
-        if (!parseArgs(args, "UO", &UTransPositionType, &u0, &utransposition))
+        if (!parseArgs(args, "UO", &UTransPositionType_, &u0, &utransposition))
         {
             STATUS_PYTHON_CALL(self->object->transliterate(*u0, *utransposition->object, status));
             Py_RETURN_ARG(args, 0);
         }
-        if (!parseArgs(args, "sO", &UTransPositionType, &_u0, &utransposition))
+        if (!parseArgs(args, "sO", &UTransPositionType_, &_u0, &utransposition))
         {
             STATUS_PYTHON_CALL(self->object->transliterate(_u0, *utransposition->object, status));
             return PyUnicode_FromUnicodeString(&_u0);
@@ -413,7 +413,7 @@ static PyObject *t_transliterator_transliterate(t_transliterator *self,
             PYTHON_CALL(self->object->transliterate(_u0, start, limit));
             return PyUnicode_FromUnicodeString(&_u0);
         }
-        if (!parseArgs(args, "UOS", &UTransPositionType,
+        if (!parseArgs(args, "UOS", &UTransPositionType_,
                        &u0, &utransposition, &u1, &_u1))
         {
             STATUS_CALL(len = toUChar32(*u1, &c, status));
@@ -427,7 +427,7 @@ static PyObject *t_transliterator_transliterate(t_transliterator *self,
             }
             Py_RETURN_ARG(args, 0);
         }
-        if (!parseArgs(args, "sOS", &UTransPositionType, 
+        if (!parseArgs(args, "sOS", &UTransPositionType_, 
                        &_u0, &utransposition, &u1, &_u1))
         {
             STATUS_CALL(len = toUChar32(*u1, &c, status));
@@ -453,12 +453,12 @@ static PyObject *t_transliterator_finishTransliteration(t_transliterator *self,
     UnicodeString *u, _u;
     t_utransposition *utransposition;
 
-    if (!parseArgs(args, "UO", &UTransPositionType, &u, &utransposition))
+    if (!parseArgs(args, "UO", &UTransPositionType_, &u, &utransposition))
     {
         self->object->finishTransliteration(*u, *utransposition->object);
         Py_RETURN_ARG(args, 0);
     }
-    if (!parseArgs(args, "sO", &UTransPositionType, &_u, &utransposition))
+    if (!parseArgs(args, "sO", &UTransPositionType_, &_u, &utransposition))
     {
         self->object->finishTransliteration(_u, *utransposition->object);
         return PyUnicode_FromUnicodeString(&_u);
@@ -474,14 +474,14 @@ static PyObject *t_transliterator_filteredTransliterate(t_transliterator *self,
     t_utransposition *utransposition;
     int incremental;
 
-    if (!parseArgs(args, "UOB", &UTransPositionType, &u, &utransposition,
+    if (!parseArgs(args, "UOB", &UTransPositionType_, &u, &utransposition,
                    &incremental))
     {
         self->object->filteredTransliterate(*u, *utransposition->object,
                                             incremental);
         Py_RETURN_ARG(args, 0);
     }
-    if (!parseArgs(args, "sOB", &UTransPositionType, &_u, &utransposition,
+    if (!parseArgs(args, "sOB", &UTransPositionType_, &_u, &utransposition,
                    &incremental))
     {
         self->object->filteredTransliterate(_u, *utransposition->object,
@@ -588,12 +588,25 @@ static PyObject *t_transliterator_createInstance(PyTypeObject *type,
 {
     Transliterator *transliterator;
     UnicodeString *u, _u;
-    UTransDirection direction;
+    UTransDirection direction = UTRANS_FORWARD;
 
-    if (!parseArgs(args, "Si", &u, &_u, &direction))
-    {
-        STATUS_CALL(transliterator = Transliterator::createInstance(*u, direction, status));
-        return wrap_Transliterator(transliterator);
+    switch (PyTuple_Size(args)) {
+      case 1:
+        if (!parseArgs(args, "S", &u, &_u))
+        {
+            STATUS_CALL(transliterator = Transliterator::createInstance(
+                *u, direction, status));
+            return wrap_Transliterator(transliterator);
+        }
+        break;
+      case 2:
+        if (!parseArgs(args, "Si", &u, &_u, &direction))
+        {
+            STATUS_CALL(transliterator = Transliterator::createInstance(
+                *u, direction, status));
+            return wrap_Transliterator(transliterator);
+        }
+        break;
     }
 
     return PyErr_SetArgsError(type, "createInstance", args);
@@ -605,12 +618,25 @@ static PyObject *t_transliterator_createFromRules(PyTypeObject *type,
     Transliterator *transliterator;
     UnicodeString *u0, _u0;
     UnicodeString *u1, _u1;
-    UTransDirection direction;
+    UTransDirection direction = UTRANS_FORWARD;
 
-    if (!parseArgs(args, "SSi", &u0, &_u0, &u1, &_u1, &direction))
-    {
-        STATUS_PARSER_CALL(transliterator = Transliterator::createFromRules(*u0, *u1, direction, parseError, status));
-        return wrap_Transliterator(transliterator);
+    switch (PyTuple_Size(args)) {
+      case 2:
+        if (!parseArgs(args, "SS", &u0, &_u0, &u1, &_u1))
+        {
+            STATUS_PARSER_CALL(transliterator = Transliterator::createFromRules(
+                *u0, *u1, direction, parseError, status));
+            return wrap_Transliterator(transliterator);
+        }
+        break;
+      case 3:
+        if (!parseArgs(args, "SSi", &u0, &_u0, &u1, &_u1, &direction))
+        {
+            STATUS_PARSER_CALL(transliterator = Transliterator::createFromRules(
+                *u0, *u1, direction, parseError, status));
+            return wrap_Transliterator(transliterator);
+        }
+        break;
     }
 
     return PyErr_SetArgsError(type, "createFromRules", args);
@@ -639,8 +665,8 @@ static PyObject *t_transliterator_str(t_transliterator *self)
 
 void _init_transliterator(PyObject *m)
 {
-    TransliteratorType.tp_str = (reprfunc) t_transliterator_str;
-    UTransPositionType.tp_getset = t_utransposition_properties;
+    TransliteratorType_.tp_str = (reprfunc) t_transliterator_str;
+    UTransPositionType_.tp_getset = t_utransposition_properties;
 
     INSTALL_CONSTANTS_TYPE(UTransDirection, m);
     INSTALL_STRUCT(UTransPosition, m);
